from pathlib import Path
from .core import *

try:
    version_path = Path(__file__).parent / "VERSION"
    version = version_path.read_text().strip()
except FileNotFoundError:
    version = "0.0.0"

__version__ = version
del version



from walle.core import UserNameType
from walle.configs import WelcomeConfig

import pytest


@pytest.mark.parametrize(
    "test_input,expected",
    [
        (
            {
                "key": "username",
                "value": "rube",
                "name": "dummy-type",
            },
            "Welcome to the server, rube!",
        ),
    ],
)
def test_eval(test_input, expected):
    got = UserNameType(**test_input)
    types = [got]
    got = WelcomeConfig(types)
    assert str(got) == expected



from base import BaseBot


class WelcomeBot(BaseBot):
    """Welcome Bot

    The purpose of this welcome bot
    will be a endpoint for any interaction involving
    events and/or actions that are associated with
    welcome to servers/channels/clans/groups/etc

    Parameters
    ----------

    action_config : ActionConfig(Config)
        A typed/defined action config that will
        have a predefined set of supported Primitive OPs

    event_config : EventConfig(Config)
        A typed/defined event config that will
        have a predefined set of supported Primitive OPs

    name : str
        the name of this specific bot

    Usage
    -----

    ```python

    from walle.configs import ActionConfig, EventConfig
    from walle.bot import WelcomeBot

    action_cfg = ActionConfig("welcome-actions")
    event_cfg = EventConfig("welcome-events")
    welcome_bot = WelcomeBot(
        action_config=action_cfg,
        event_cfg=event_cfg,
        name=name,
    )
    ```

    Notes
    -----

        The idea of config classes may change as we develop
            the basic idea seems sounds, but could be an entire rewrite

        We dont have a firm grasp on the scope of each bot, this is
            one direction we can go with it
    """

    def __init__(
        self,
        action_config,
        event_config,
        name="welcome-bot",
    ):
        super().__init__(
            action_config=action_config,
            event_config=event_config,
            name=name,
        )



from abc import ABCMeta, abstractmethod

__all__ = [
    "BaseBot",
]


class BaseBot(metaclass=ABCMeta):
    """Base Bot"""

    def __init__(
        self,
        event_config,
        action_config,
        name="base-bot",
    ):
        self.event_config = event_config
        self.action_config = action_config
        self.name = name
        if name.contains("base"):
            with AssertionError as msg:
                print(msg, "Expected new bot class name but got: ", self.name)

    @abstractmethod
    def handle_event(self, event):
        """ """
        pass

    @abstractmethod
    def handle_action(self, action):
        """ """
        pass

    @abstractmethod
    def _parse_event_tokens(self, tokens, **kwargs):
        """ """
        pass

    @abstractmethod
    def _parse_action_tokens(self, tokens, **kwargs):
        """ """
        pass
